/*
 *************************************************************************
 * Copyright (c) 2004, 2005 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *  
 *************************************************************************
 */

package org.eclipse.datatools.connectivity.oda.consumer.helper;

import java.util.Locale;

import org.eclipse.datatools.connectivity.oda.OdaException;
import org.eclipse.datatools.connectivity.oda.consumer.i18n.DataResourceHandle;

/**
 * Exception thrown for errors that occur within the 
 * ODA consumer helper package.  It encapsulates the handling
 * of localized error messages.
 */
public class OdaHelperException extends OdaException
{
	// Maps from old ResourceConstants IDs to the index of current resource keys
	private static String[] sm_resourceKeys = 
	{
		"oda.consumer.helper.CannotGetNamedResultsAfterExecuteQuery", // CANNOT_GET_NAMED_RS_AFTER_EXECUTEQUERY
		"oda.consumer.helper.CannotSetParamBeforePrepare", // CANNOT_SET_PARAM_BEFORE_PREPARE
		"oda.consumer.helper.CannotGetParamBeforeExecute", // CANNOT_GET_PARAM_BEFORE_EXECUTE
		"oda.consumer.helper.CannotGetParamMdBeforePrepare", // CANNOT_GET_PARAMETER_METADATA_BEFORE_PREPARE
		"oda.consumer.helper.CannotSetSortSpecBeforePrepare", // CANNOT_SET_SORT_SPEC_BEFORE_PREPARE
		"oda.consumer.helper.MaxConcurrentConnectionsReached", // MAX_CONCURRENT_CONNECTIONS_REACHED
		"oda.consumer.helper.ConnectionFailed", // CONNECTION_FAILED
		"oda.consumer.helper.CannotConstructConnectionFactory", // CANNOT_CONSTRUCT_CONNECTION_FACTORY
		"oda.consumer.helper.InadequatePermissionsForCompatibility", // INADEQUATE_PERMISSION_FOR_COMPATIBILITY
		"oda.consumer.helper.ConnectionIsInactive", // ESTABLISTHED_CONNECTION_NOT_ACTIVE
		"oda.consumer.helper.MaxConcurrentStatementsReached", // MAX_CONCURRENT_STATEMENTS_REACHED
		"oda.consumer.helper.PrepareFailed", // PREPARE_FAILED
		"oda.consumer.helper.CannotGetResultSetMdBeforePrepare", // CANNOT_GETMETADATA_BEFORE_PREPARE
		"oda.consumer.helper.CannotExecuteBeforePrepare" // CANNOT_EXECUTE_BEFORE_PREPARE
	};
	
	private int m_errorNumber;
	private Locale m_locale;
	private String m_appendInfo;
	
	public OdaHelperException( int errorNumber, Locale locale, String appendInfo )
	{
		assert( errorNumber >= 0 && errorNumber < sm_resourceKeys.length );
		
		m_errorNumber = errorNumber;
		m_locale = locale;
		m_appendInfo = appendInfo;
	}
	
	public String getMessage()
	{
		Locale defaultLocale = Locale.US;
		return getLocalizedMessageImpl( defaultLocale );
	}
	
	public String getLocalizedMessage()
	{
		return getLocalizedMessageImpl( m_locale );
	}
	
	private String getLocalizedMessageImpl( Locale locale )
	{
		String key = sm_resourceKeys[ m_errorNumber ];
		
		assert( key != null );
		String msg = DataResourceHandle.getInstance( locale ).getMessage( key );
		
		assert( msg != null && msg.length() > 0 );
		
		return msg;
	}
	
	public String toString()
	{
		String ret = getClass().getName();
		String localizedMessage = getLocalizedMessage();
		if( localizedMessage != null && localizedMessage.length() > 0 )
			ret += ": " + localizedMessage;
		
		if( m_appendInfo != null && m_appendInfo.length() > 0 )
			ret += " (" + m_appendInfo + ")";
		
		Throwable cause = getCause();
		if( cause != null )
		{
			String causeString = cause.toString();
			if( causeString != null && causeString.length() > 0 )
				ret += " ;\n" + causeString;
		}
		
		return ret;
	}
}
