/*
 *************************************************************************
 * Copyright (c) 2004, 2005 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *  
 *************************************************************************
 */

package org.eclipse.datatools.connectivity.oda.consumer.i18n;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;

import com.ibm.icu.util.ULocale;
import com.ibm.icu.util.UResourceBundle;

/**
 * Implementation of ResourceHandle in ODA Consumer helper.
 */

public class DataResourceHandle
{
	private static DataResourceHandle sm_resourceHandle;
	private static Map sm_localeResourceHandleMap;

    /**
     * The actual resource bundle. The implementation assumes that Java will use
     * a PropertyResourceBundle to access our files.
     */
    private UResourceBundle m_resources;

    // Name of the resource bundle.
    private final static String BUNDLE_NAME = "Messages"; //$NON-NLS-1$
	
	static DataResourceHandle getInstance()
	{
		if( sm_resourceHandle == null )
			sm_resourceHandle = new DataResourceHandle( ULocale.getDefault() );
		
		return sm_resourceHandle;
	}
	
	public static DataResourceHandle getInstance( Locale locale )
	{
		DataResourceHandle ret = 
			(DataResourceHandle) getLocaleResourceHandleMap().get( locale );
		
		if( ret == null )
		{
			ret = new DataResourceHandle( locale );
			getLocaleResourceHandleMap().put( locale, ret );
		}
		
		return ret;
	}
	
	private static Map getLocaleResourceHandleMap()
	{
		if( sm_localeResourceHandleMap == null )
			sm_localeResourceHandleMap = new HashMap();
		
		return sm_localeResourceHandleMap;
	}

	/**
	 * @param locale
	 */
	DataResourceHandle( Locale locale )
	{
        this( ULocale.forLocale( locale ) );
	}

    /**
     * Constructor.
     * 
     * @param locale
     *            the user's locale. If null, the default locale for the JVM
     *            will be used.
     */

    DataResourceHandle( ULocale locale )
    {
        String className = this.getClass( ).getName( );
        String bundleName = ""; //$NON-NLS-1$

        // Create the base message file name formatted like a Java class.
        // The Java class loader will search for the file using the same
        // algorithm used to find classes.

        int index = className.lastIndexOf( '.' );
        if ( index > -1 )
        {
            // e.g: "org.eclipse.birt.report.model.util.Test"

            bundleName = className.substring( 0, index ) + "."; //$NON-NLS-1$
        }

        bundleName = bundleName + BUNDLE_NAME;
        if ( locale == null )
            locale = ULocale.getDefault( );
        m_resources = UResourceBundle.getBundleInstance( bundleName, locale, this.getClass().getClassLoader() );
        assert m_resources != null : "ResourceBundle : " + BUNDLE_NAME + " for " + locale + " not found"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
    }

    /**
     * Get a message given the message key. An assertion will be raised if the
     * message key does not exist in the resource bundle.
     * @param key the message key
     * @return the localized message for that key and the locale set in the
     *         constructor. Returns the key itself if the message was not found.
     * @see UResourceBundle#getString( String )
     */
    public String getMessage( String key )
    {
        if ( key == null )
            return null;
        try
        {
            return m_resources.getString( key );
        }
        catch ( MissingResourceException e )
        {
            // It is a programming error to refer to a missing
            // message.
            assert false : key + " not found in resource bundle"; //$NON-NLS-1$
            return key;
        }
    }

}
