/*******************************************************************************
 * Copyright (c) 2004, 2005 Sybase, Inc. and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Sybase, Inc. - initial API and implementation
 *******************************************************************************/

/*
 * Created on 2004-6-16 Copyright Sybase, Inc. All rights reserved.
 */
package org.eclipse.datatools.sqltools.debugger.core;

import java.sql.Connection;
import java.sql.SQLException;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.datatools.sqltools.core.ConnectionException;
import org.eclipse.datatools.sqltools.core.IControlConnection;
import org.eclipse.datatools.sqltools.core.ProcIdentifier;
import org.eclipse.datatools.sqltools.core.profile.NoSuchProfileException;


/**
 * The "debugger control connection" is designed for the following
 * functions: <LL>
 * <li>Used for debugging. 
 * <li>Manage all client connection to the server (debugged or not debugged), the connections view will display these
 * information.
 * 
 * @author Yang Liu
 * @author Hui Cao
 */
public interface IDebuggerControlConnection extends IAdaptable, IControlConnection
{
    /**
     * Registers the debuggee connection identified by <code>connId</code>. Called by the debugger to information this controller that the specified connection is being
     * debugged. if the same id debuggee already registered, then the behavior is undefined.
     * 
     * @param connId the database specific connection id @see org.eclipse.datatools.sqltools.core.services.IConnectionService#getConnectionId(org.eclipse.datatools.sqltools.core.DatabaseIdentifier, java.sql.Connection)
     */
    public void registerDebuggee(String connId, IConnectionObserver observer)
        throws ConnectionException;

    /**
     * Unregisters the debuggee connection identified by <code>connId</code>. This method is called when the debuggee is no longer being debugged.
     * 
     * @param connId the database specific connection id @see org.eclipse.datatools.sqltools.core.services.IConnectionService#getConnectionId(org.eclipse.datatools.sqltools.core.DatabaseIdentifier, java.sql.Connection)
     */
    public void unregisterDebuggee(String connId);

    /**
     * Checks whether there is registered debuggee.
     * @return true if any; false if none.
     */
    public boolean hasDebuggee();

    /**
	 * Requests to stop debugging on the specified connection
     * @param connId the database specific connection id @see org.eclipse.datatools.sqltools.core.services.IConnectionService#getConnectionId(org.eclipse.datatools.sqltools.core.DatabaseIdentifier, java.sql.Connection)
	 */
    public void requestDetach(String connId);

    /**
     * Tests whether the specified connection is attached by debugger
     * 
     * @param connId the database specific connection id @see org.eclipse.datatools.sqltools.core.services.IConnectionService#getConnectionId(org.eclipse.datatools.sqltools.core.DatabaseIdentifier, java.sql.Connection)
     */
    public boolean isAttached(String connId);

    /**
     * Gets the client connection information.
     * 
     * @return <code>ClientConInfo</code> array managed by this control connection.
     */
    public ClientConInfo[] getClientConInfos();

    /**
     * Get the client connection information by id.
     * 
     * @param connId the database specific connection id @see org.eclipse.datatools.sqltools.core.services.IConnectionService#getConnectionId(org.eclipse.datatools.sqltools.core.DatabaseIdentifier, java.sql.Connection)
     * @return
     */
    public ClientConInfo getClientConInfo(String connId);

    /**
     * @see #getClientConInfos()
     * @return
     */
    public boolean hasClientConnection();

    /**
     * Refreshs external clients info. This may be called from UI showing external clients info.
     *
     */
    public void refreshExternalClients();


    /**
     * Given the ProcIdentifier and the line numbers, finds out the valid line numbers that is 
     * greater than number that can set breakpoint.
     * 
     * If for some reason, this operation is not supported, the original number will be returned.
     * @param proc the routine object containing the breakpoints
     * @param lineNumbers original breakpoint line numbers
     * @return line numbers. Valid numbers are greater or equals to 0. -1 means can't find a valid location.
     */
    public int[] getValidBreakpointLocations(ProcIdentifier proc, int[] lineNumbers) throws SQLException;

    /**
     * Checks whether ths control connection is ready to debug. Return null if it is ready, otherwise, return the error message.
     * @return error message or null
     */
    public String readyToDebug();
    
    /**
     * Creates a debuggee connection.
     * If can't get connection, will put "" as the connid.
     * 
     * @param connId	the connection id of the new connection.
     * @return
     * @throws NoSuchProfileException
     * @throws CoreException
     * @throws SQLException
     */
    public Connection createConnection(String[] connId) throws SQLException, CoreException, NoSuchProfileException;


}
 