/*******************************************************************************
 * Copyright (c) 2004, 2005 Sybase, Inc. and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Sybase, Inc. - initial API and implementation
 *******************************************************************************/

/*
 * Created on 2004-6-16 Copyright Sybase, Inc. All rights reserved.
 */
package org.eclipse.datatools.sqltools.debugger.core.internal;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.datatools.sqltools.core.ConnectionException;
import org.eclipse.datatools.sqltools.core.DatabaseIdentifier;
import org.eclipse.datatools.sqltools.core.ProcIdentifier;
import org.eclipse.datatools.sqltools.core.dbitem.IDBItem;
import org.eclipse.datatools.sqltools.core.dbitem.IItemWithCode;
import org.eclipse.datatools.sqltools.debugger.core.ClientConInfo;
import org.eclipse.datatools.sqltools.debugger.core.IConnectionObserver;
import org.eclipse.datatools.sqltools.debugger.core.IDebuggerControlConnection;
import org.eclipse.datatools.sqltools.internal.core.AbstractControlConnection;
import org.eclipse.datatools.sqltools.internal.core.ControlConnectionManager;


/**
 * This class is used as base class for ASAControlConnection and ASEControlConnection.
 * 
 * @author Yang Liu
 */
public abstract class AbstractDebuggerControlConnection extends AbstractControlConnection implements IDebuggerControlConnection
{
    /**
     * id -> ClientConInfo mapping.
     */
    protected Map           _clientConnections = new TreeMap();

    /**
     * id -> IConnectionObserver mapping. Only for those attached debuggee connections.
     */
    protected Map           _debuggerObservers = new TreeMap();


    /**
     *  
     */
    public AbstractDebuggerControlConnection(ControlConnectionManager manager, DatabaseIdentifier databaseIdentifier)
    {
        super(manager, databaseIdentifier);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sybase.stf.dmp.core.IControlConnection#getClientConInfos()
     */
    public ClientConInfo[] getClientConInfos()
    {
        return (ClientConInfo[]) _clientConnections.values().toArray(new ClientConInfo[_clientConnections.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sybase.stf.dmp.core.IControlConnection#getClientConInfo(int)
     */
    public ClientConInfo getClientConInfo(int connid)
    {
        return (ClientConInfo) _clientConnections.get(new Integer(connid));
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sybase.stf.dmp.core.IControlConnection#registerSkip(int)
     */
    public void registerSkip(int connid)
    {
        Integer id = new Integer(connid);
        _skipConnections.add(id);
        if (_clientConnections.remove(id) != null)
        {
            fireChange();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sybase.stf.dmp.core.IControlConnection#hasClientConnection()
     */
    public boolean hasClientConnection()
    {
        return !_clientConnections.isEmpty();
    }

    public boolean isAttached(int connid)
    {
        return _debuggerObservers.containsKey(new Integer(connid));
    }

    public boolean hasDebuggee()
    {
        return !_debuggerObservers.isEmpty();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sybase.stf.dmp.core.IControlConnection#registerDebuggee(int,
     *      com.sybase.stf.dmp.core.IConnectionObserver)
     */
    public void registerDebuggee(int id, IConnectionObserver observer) throws ConnectionException
    {
        if (_debuggerObservers.get(new Integer(id)) != null)
        {
            throw new ConnectionException(ConnectionException.DUPLICATE_DEBUGGEE, DebuggerMessages.getString("AbstractControlConnection.connection.already.being.debugged", String.valueOf(id))); //$NON-NLS-1$ 
        }
        _debuggerObservers.put(new Integer(id), observer);
        fireChange();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.sybase.stf.dmp.core.IControlConnection#unregisterDebuggee(int)
     */
    public void unregisterDebuggee(int id)
    {
        if (_debuggerObservers.remove(new Integer(id)) != null)
        {
            fireChange();
        }
    }

    /* (non-Javadoc)
	 * @see com.sybase.stf.dmp.core.IControlConnection#requestDetach(int)
	 */
    public void requestDetach(String connId) 
    {
        IConnectionObserver observer = this.getObservable(connId);
        if (observer != null) observer.requestDisconnect(connId);
    }

    protected IConnectionObserver getObservable(String connid)
    {
        return (IConnectionObserver) this._debuggerObservers.get(new Integer(connid));
    }

    public void fireChange()
    {
        _manager.fireRefreshed(this);
    }

    /**
     * dispose the resources.
     * Child class can override this method, but normally should call super.dispose()
     */
    protected void dispose()
    {
        // inform all the observers that we are shutting down.
        // create a copy, because during observableShutdown the observer set may change.
        List set = new ArrayList(_debuggerObservers.values());
        for (Iterator iter = set.iterator(); iter.hasNext();)
        {
            IConnectionObserver observer = (IConnectionObserver) iter.next();
            observer.observableShutdown();
        }
    }

    public boolean okToDisconnect()
    {
        return !hasDebuggee();
    }

    public boolean disconnect()
    {
        return disconnect(false);
    }

    public boolean disconnect(boolean force)
    {
        // XXX: temp implementation
        if (hasDebuggee() && !force)
        {
            return false;
        }
        return super.disconnect(force);
    }

    /* (non-Javadoc)
     * @see com.sybase.stf.dmp.core.IControlConnection#getValidBreakpointLocations(com.sybase.stf.dmp.core.ProcIdentifier, int[])
     */
    public int[] getValidBreakpointLocations(ProcIdentifier proc, int[] lineNumbers) throws SQLException
    {
        IDBItem item = getDBItem(proc);
        if (item instanceof IItemWithCode)
        {
            int[] ret = new int[lineNumbers.length];
            IItemWithCode codeItem = (IItemWithCode) item;
            for (int i=0; i<lineNumbers.length; i++) ret[i] = codeItem.getValidBreakpointLocation(lineNumbers[i]); 
            return ret;
        }
        else
        {
            // should not happen
            return lineNumbers;
        }
    }

    public void refreshExternalClients()
    {
        // Do noting by default since not all vendors need to call this API explicitly
    }

    /* (non-Javadoc)
     * @see com.sybase.stf.dmp.core.IControlConnection#supportDebug()
     */
    public String readyToDebug()
    {
        return null;
    }

    public boolean supportsDebugging()
    {
        return true;
    }


}
